package com.prosc.commerce.gateway.linkpoint;

import com.prosc.commerce.gateway.PaymentAuthorizationException;
import com.prosc.commerce.gateway.PaymentGateway;
import com.prosc.commerce.gateway.PaymentIOException;
import lp.order.LPOrderFactory;
import lp.order.LPOrderPart;
import lp.txn.JLinkPointTransaction;
import sun.security.validator.ValidatorException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Note: This class does not implement <items><option></option></items> entity/sub-entity when
 * processing payment transactions.  TODO Items/Option Entity/Sub-Entity processing?  --svr
 * @author sbarnum
 */
public class LinkpointPaymentGateway extends PaymentGateway {
    private static final Logger log = Logger.getLogger(LinkpointPaymentGateway.class.getName());

    private static final String PKCS12_ALIAS = "linkpoint";

    public static final String TXTYPE_SALE = "SALE";
    public static final String TXTYPE_CREDIT = "CREDIT";
    public static final String TXTYPE_VOID = "VOID";
    public static final String TXTYPE_PREAUTH = "PREAUTH";
    public static final String TXTYPE_POSTAUTH = "POSTAUTH";

    public static final String ACTION_SUBMIT = "SUBMIT";
    public static final String ACTION_MODIFY = "MODIFY";
    public static final String ACTION_CANCEL = "CANCEL";

    public static final char TENDER_TELECHECK = 'E';
    public static final char TENDER_CREDITCARD = 'C';

    public static final String VOID_TELECHECK = "1";

    public static final String TELECHECK_ACCT_TYPE_PERSONAL_CHECKING = "PC";
    public static final String TELECHECK_ACCT_TYPE_PERSONAL_SAVINGS = "PS";
    public static final String TELECHECK_ACCT_TYPE_BUSINESS_CHECKING = "BC";
    public static final String TELECHECK_ACCT_TYPE_BUSINESS_SAVINGS = "BS";

    public static final int RECUR_INFINITE_NUM_INSTALLMENTS = 99;

    private static final Format recurringStartDateFormat = new SimpleDateFormat("yyyyMMdd");

    /**
     * Indicates whether transaction is recurring
     */
    private Boolean isRecurring = Boolean.FALSE;

    /*
   //////////////////////////
   //CUSTOMER INPUT VALUES
   //////////////////////////
    */

    /*
    //////////////////////////
    //<payment>
    //////////////////////////
     */
    private BigDecimal subtotal;
    private BigDecimal tax;
    private BigDecimal shipping;

    /*
    Amount of VAT (typically, only tax or vattax is used).
    This is a tax sometimes applied to international orders.
     */
//    private BigDecimal vattax;  //TODO Should this be part of our API? --svr

    /*
    //////////////////////////
    //<creditcard>
    //////////////////////////
     */
    /*
        Authenticate value required for Verified By Visa and MasterCard SecureCode protection.
        Value is returned from FirstData MPI
    */
//    private String cavv;  //TODO Should this be part of our API? --svr

    /*
        Authenticate value required for Verified By Visa and MasterCard SecureCode protection.
        Value is returned from FirstData MPI
    */
//    private String xid;  //TODO Should this be part of our API? --svr

    /*
        Electronic Commerce Indicator Value is returned from FirstData MPI
    */
//    private String eci;  //TODO Should this be part of our API? --svr

    /*
    //////////////////////////
    //<telecheck>
    //////////////////////////
     */
    private String bankState;
    private String driversLicenseNumber;
    private String driversLicenseState;



    /*
   //////////////////////////
   //<billing>
   //////////////////////////
    */
    private String companyName;
    private String fax;

    /*
    //////////////////////////
    //<com.prosc.shipping>
    //////////////////////////
     */
    private BigDecimal weight;
    private int items;
    private int carrier;
    private BigDecimal total;


    /*
    //////////////////////////
    //<transactiondetails>
    //////////////////////////
     */
    /*
    This field is returned with every successful transaction.
    If you need to run a void or other transaction against an existing order ID, you may need to pass the tdate field
    to uniquely identify the specific transaction you want to void.
     */
    private int tdate;

    /*
    //////////////////////////
    //<notes>
    //////////////////////////
     */
    private String referred;

    ////////////////////////////////////////////////////////////////////////////////

    private String transactionType = TXTYPE_SALE;	//The default transaction type
    private String recurringTransactionType = ACTION_SUBMIT;    //The default recurring transaction type
    private boolean testMode = false;
    protected String host;
    protected int port = 1129;
    private File certPath;
    //    private KeyStore tempKeyStore;
    private int chargeResult;
    private String tender;

    private String debugInfo;

    public LinkpointPaymentGateway() {
        tender = String.valueOf(TENDER_CREDITCARD);
    }

    protected void _processPayment() throws IllegalArgumentException, PaymentIOException, PaymentAuthorizationException {

        //Determines which server gets request
        host = testMode ? "staging.linkpt.net" : "secure.linkpt.net";
//        host = testMode ? "www.staging.linkpointcentral.com" : "secure.linkpt.net";

        KeyStore keyStore = null;
        if (testMode) {
            log.log(Level.INFO, "Adding host server's certificate to trusted keystore");
            try {
                keyStore = storeHostCertificate("resource/firstdata.pem", PKCS12_ALIAS);
            } catch (Exception e) {
                log.log(Level.SEVERE, "An error occurred while trying to add the host server's certificate to the trusted keystore: " + e.getMessage(), e);
                throw new PaymentAuthorizationException(e.getMessage());
            }
        }

        JLinkPointTransaction txn = new JLinkPointTransaction();

        if (certPath == null) {
            throw new PaymentAuthorizationException("Digital certificate not set");
        }

		if (!certPath.exists()) {
			throw new PaymentAuthorizationException("Digital certificate does not exist at " + certPath.getAbsolutePath());
		}

		if (!certPath. canRead()) {
			throw new PaymentAuthorizationException("Unable to read digital certificate at " + certPath.getAbsolutePath());
		}


//        txn.setClientCertificatePath("/var/folders/yB/yBJX9k9jGH8E3FPo8RhdyU+++TM/-Tmp-/360Works8209182251452623095p12");
        String path = this.getCertPath().getAbsolutePath();
        txn.setClientCertificatePath(path);
        txn.setPassword(getTransactionKey());
        txn.setHost(host);
        txn.setPort(port);

        String sResponse = "";

        try {
//            String xml = getXml();
            String xml = buildXml(isRecurring.booleanValue());
            log.log(Level.FINE, "Sending XML request message to " + host + ":" + port + xml);
            debugInfo = "";
            debugInfo += "Sending XML request message to " + host + ":" + port + xml;
            sResponse = txn.send(xml);
            log.log(Level.INFO, "Receiving XML response from " + host + ":" + port + "\n" + sResponse);
            debugInfo += "\n\"Receiving XML response from \" + host + \":\" + port + \"\\n\"" + sResponse;
        } catch (ValidatorException e) {
            throw new PaymentAuthorizationException(e.getMessage());
        } catch (Exception e) {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new PaymentAuthorizationException(e.getMessage());
        } finally {
            if (keyStore != null) {
                try {
                    keyStore.deleteEntry(PKCS12_ALIAS);
                } catch (KeyStoreException e) {
                    String errMsg = "An error occurred while trying to delete an entry with alias " + PKCS12_ALIAS + " from the keystore";
                    log.log(Level.SEVERE, errMsg, e);
//                    throw new PaymentAuthorizationException(errMsg + ": " + e.getMessage());
                }
            }
        }

//        if (sResponse.equals("")) {
//            throw new PaymentAuthorizationException("SGS-020005: Error (Merchant config file is missing, empty or cannot be read).");
//        }

        parseResponse(sResponse);

//        if ("APPROVED".equals(R_Approved)) {
//            chargeResult = 1;
//        } else if ("DECLINED".equals(R_Approved)) {
//            chargeResult = 2;
//        } else { //"FRAUD".equals(R_Approved)
//            chargeResult = 3;
//        }

        if (!"APPROVED".equals(R_Approved)) {
            String msg = R_Error != null ? R_Error : R_Message;
            if (msg == null) {
                msg = "Unknown error";
                if (sResponse.equals("")) msg+= "\n\nOne possible cause of this error is an attempt to access the live server with" +
                        " a test account and its associated certificate";
            }

            throw new PaymentAuthorizationException(R_Approved + ": " + msg);
        }
    }

    protected void _processRecurringPayment() throws IllegalArgumentException, PaymentIOException, PaymentAuthorizationException {
        isRecurring = Boolean.TRUE;
        _processPayment();
    }

    public String getDebugInfo() {
        return debugInfo;
    }

    protected void parseResponse(String rsp) {
        R_Time = parseTag("r_time", rsp);
        R_Ref = parseTag("r_ref", rsp);
        R_Approved = parseTag("r_approved", rsp);
        R_Code = parseTag("r_code", rsp);
        R_Authresp = parseTag("r_authresponse", rsp);
        R_Error = parseTag("r_error", rsp);
        R_OrderNum = parseTag("r_ordernum", rsp);
        R_Message = parseTag("r_message", rsp);
        R_Score = parseTag("r_score", rsp);
        R_TDate = parseTag("r_tdate", rsp);
        R_AVS = parseTag("r_avs", rsp);
        R_Tax = parseTag("r_tax", rsp);
        R_Shipping = parseTag("r_shipping", rsp);
        R_FraudCode = parseTag("r_fraudCode", rsp);
        R_ESD = parseTag("esd", rsp);
    }

    public void printResp() {
        System.out.println("\n******  XML RESPONSE  *******");
        if (R_Time!= null)
            System.out.println("<r_time>" + R_Time + "</r_time>");
        if (R_Ref!= null)
            System.out.println("<r_ref>" + R_Ref + "</r_ref>");
        if (R_Approved!= null)
            System.out.println("<r_approved>" + R_Approved + "</r_approved>");
        if (R_Code!= null)
            System.out.println("<r_code>" + R_Code + "</r_code>");
        if (R_OrderNum!= null)
            System.out.println("<r_ordernum>" + R_OrderNum + "</r_ordernum>");
        if (R_Error!= null)
            System.out.println("<r_error>" + R_Error + "</r_error>");
        if (R_FraudCode!= null)
            System.out.println("<r_fraudCode>" + R_FraudCode + "</r_fraudCode>");
        if (R_Authresp != null)
            System.out.println("<r_authresponse>" + R_Authresp + "</r_authresponse>");
        if (R_Message!= null)
            System.out.println("<r_message>" + R_Message + "</r_message>");
        if (R_TDate!= null)
            System.out.println("<r_tdate>" + R_TDate + "</r_tdate>");
        if (R_Shipping!= null)
            System.out.println("<r_shipping>" + R_Shipping + "</r_shipping>");
        if (R_Tax!= null)
            System.out.println("<r_tax>" + R_Tax + "</r_tax>");
        if (R_AVS!= null)
            System.out.println("<r_avs>" + R_AVS + "</r_avs>");
        if (R_ESD!= null)
            System.out.println("<esd>" + R_ESD + "</esd>");
        if (R_Score!= null)
            System.out.println("<r_score>" + R_Score + "</r_score>");

        System.out.println("\n****** End Of XML RESPONSE ********");

    }

    // response holders
    /**
     * The time and date of the transaction server response.
     */
    protected String R_Time = null;
    /**
     * The reference number returned by the credit card processor.
     */
    protected String R_Ref = null;
    /**
     * The result of the transaction, which may be APPROVED, DECLINED, or FRAUD.
     */
    protected String R_Approved = null;
    /**
     * The approval code for this transaction.
     */
    protected String R_Code = null;
    protected String R_Authresp = null;
    /**
     * Any error message associated with this transaction.
     */
    protected String R_Error = null;
    /**
     * The order number associated with this transaction.
     */
    protected String R_OrderNum = null;
    /**
     * Any message returned by the processor; e.g., CALL VOICE CENTER.
     */
    protected String R_Message = null;
    protected String R_Score = null;
    /**
     * A server time-date stamp for this transaction. Used to uniquely identify a specific transaction where one order
     * number may apply to several individual transactions. See the Transaction Details Data Fields section for further
     * information and an example of tdate.
     */
    protected String R_TDate = null;
    /**
     * The Address Verification System (AVS) response for this transaction. The first character indicates whether the
     * contents of the addrnum tag match the address number on file for the billing address. The second character indicates
     * whether the billing zip code matches the billing records. The third character is the raw AVS response from the
     * card-issuing bank. The last character indicates whether the cvmvalue was correct and may be M for Match, N for
     * No Match, or Z if the match could not determined. See the sections entitled Using Address Verification and Using
     * the Card Code for additional information on using this information to help you combat fraud.
     */
    protected String R_AVS = null;
    protected String R_FraudCode = null;
    protected String R_ESD = null;
    /**
     * The calculated tax for the order, when the ordertype is calctax.
     */
    protected String R_Tax = null;
    /**
     * The calculated shipping charges for the order, when the ordertype is calcshipping.
     */
    protected String R_Shipping = null;

    protected String parseTag(String tag, String rsp) {
        StringBuffer sb = new StringBuffer(256);
        sb.append('<').append(tag).append('>');
        int len = sb.length();
        int idxSt = -1, idxEnd = -1;
        if (-1 == (idxSt = rsp.indexOf(sb.toString()))) {
            return null;
        }
        idxSt += len;
        sb.setLength(0);
        sb.append("</").append(tag).append('>');
        if (-1 == (idxEnd = rsp.indexOf(sb.toString(), idxSt))) {
            return null;
        }
        return rsp.substring(idxSt, idxEnd);
    }



    private String buildXml(boolean buildXmlForRecurring) {
//        MyOrderPart order = new MyOrderPart("order");
        LPOrderPart order = LPOrderFactory.createOrderPart("order");

        //Build <merchantinfo>
        LPOrderPart orderPart = new MyOrderPart();
//        LPOrderPart orderPart = LPOrderFactory.createOrderPart();
        orderPart.put("configfile", login());
        order.addPart("merchantinfo", orderPart);
        orderPart.clear();

        //Build <orderoptions>
        orderPart.put("ordertype", getTransactionType());
        orderPart.put("result", "LIVE");
        order.addPart("orderoptions", orderPart);
        orderPart.clear();

        //Build <payment>
        orderPart = LPOrderFactory.createOrderPart();
        orderPart.put("chargetotal", isRecurring.booleanValue() && recurringTransactionType.equals(ACTION_CANCEL) ? "1.00" : String.valueOf(amount()));
//        orderPart.put("subtotal", getSubtotal().toString());
//        orderPart.put("tax", getTax().toString());
//        orderPart.put("com.prosc.shipping", getShipping().toString());
        order.addPart("payment", orderPart);
        orderPart.clear();

        switch (tender.charAt(0)) {
            case TENDER_CREDITCARD:
                //Build <creditcard>
                orderPart.put("cardnumber", cardNumber());
                orderPart.put("cardexpmonth", expirationMonth());
                orderPart.put("cardexpyear", expirationYear());
                orderPart.put("cvmvalue", cardVerificationValue());
                orderPart.put("cvmindicator", cardVerificationValue() == null ? "not_provided" : "provided");
                orderPart.put("track", getTrack1() == null ? getTrack2() : getTrack1());
                //TODO Enter optional <payment> parameters here --svr
                order.addPart("creditcard", orderPart);
                orderPart.clear();
                break;
            case TENDER_TELECHECK:
                //Build <telecheck>
                orderPart.put("account", getAccountNumber());
                orderPart.put("routing", getAbaCode());
//                orderPart.put("checknumber", "");
                orderPart.put("bankname", getBankName());
                orderPart.put("bankstate", getBankState());
                orderPart.put("dl", getDriversLicenseNumber());
                orderPart.put("dlstate", getDriversLicenseState());
                if (getTransactionType().equals(TXTYPE_VOID)) {
                    orderPart.put("void", VOID_TELECHECK);
                }
                orderPart.put("accounttype", getAccountType());
//                orderPart.put("ssn", "");
                order.addPart("telecheck", orderPart);
                orderPart.clear();
                break;
            default:
                break;

        }

        //

        //Build <billing>
        orderPart.put("name", cardholderName());
        orderPart.put("company", getCompanyName());
        orderPart.put("address1", address1());
        orderPart.put("address2", address2());
        orderPart.put("city", city());
        orderPart.put("state", state());
        orderPart.put("zip", zip());
        orderPart.put("country", country());
        orderPart.put("phone", phone());
        orderPart.put("fax", getFax());
        orderPart.put("email", cardholderEmail());
        orderPart.put("addrnum", removeNonIntegerChars(streetAddress()));
        orderPart.put("userid", customerID());
        order.addPart("billing", orderPart);
        orderPart.clear();

        //Build <com.prosc.shipping>
        orderPart.put("name", shipFullName());
        orderPart.put("company", shipCompanyName());
        orderPart.put("address1", shipAddress());
        orderPart.put("city", city());
        orderPart.put("state", shipState());
        orderPart.put("zip", shipZip());
        orderPart.put("country", shipCountry());
        orderPart.put("phone", shipPhone());
        orderPart.put("email", customerEmail());
//        orderPart.put("weight", getWeight().toString());
//        orderPart.put("items", String.valueOf(getItems()));
//        orderPart.put("carrier", String.valueOf(getCarrier()));
//        orderPart.put("total", getTotal().toString());
        orderPart.addPart("com.prosc.shipping", orderPart);
        orderPart.clear();

        // Build <transactiondetails>
        /**
         * The source of the transaction.
         * The possible values are ECI (if the order was received via e-mail or Internet),
         * MAIL (mail order), MOTO (mail order / telephone order),  TELEPHONE (telephone order) and RETAIL (face to face).
         * For credit card transactions, mail order and telephone order transactions are treated the same.
         * However, for VirtualCheck transactions, mail order and telephone order transactions are treated differently
         * and the merchant should take care to identify them separately. Required for retail, mail, and telephone orders.
         * Defaults to ECI.
         */
        orderPart.put("transactionorigin", isCardPresent() ? "RETAIL" : "ECI");

        if (transactionType.equals(TXTYPE_CREDIT) || transactionType.equals(TXTYPE_POSTAUTH) || transactionType.equals(TXTYPE_VOID)) {
            orderPart.put("oid", getPreviousTransactionId());
        } else if (isRecurring.booleanValue() && recurringTransactionType.equals(ACTION_CANCEL)) {
            orderPart.put("oid", getRecurringPreviousSubscriptionId());
        }

        orderPart.put("taxexempt", "N");
        orderPart.put("ponumber", poNumber());
        orderPart.put("invoice_number", invoiceNumber());
        orderPart.put("terminaltype", "UNSPECIFIED");

        /**
         * The IP address of the consumer Not required, but needed for fraud blocking by IP address.
         */
        //orderPart.put("ip", "123.123.123.123"); //TODO Should this be a part of our API?  --svr

        order.addPart("transactiondetails", orderPart);
        orderPart.clear();

        if (buildXmlForRecurring) {
            addRecurringPartsToXML(order, orderPart);
        }

        //Build <notes>
        orderPart.put("comments", chargeDescription());
        orderPart.put("referred", getReferred());
        order.addPart("notes", orderPart);

        return order.toXML();
    }

    private void addRecurringPartsToXML(LPOrderPart order, LPOrderPart orderPart) {
        orderPart.clear();

        //Build <periodic>
        orderPart.put("action", getRecurringTransactionType());
        orderPart.put("installments", recurringTransactionType.equals(ACTION_CANCEL) ? "99" : String.valueOf(getRecurringNumberOfInstallments()));
        orderPart.put("periodicity", recurringTransactionType.equals(ACTION_CANCEL) ? "m1" : getPeriodicity());
        orderPart.put("startdate", recurringTransactionType.equals(ACTION_CANCEL) ? "20151231" : getRecurringSubscriptionStartDate());
        orderPart.put("comments", chargeDescription());
        orderPart.put("threshold", "3");

        order.addPart("periodic", orderPart);
        orderPart.clear();

//        order.removePart("billing");
    }

    private String getPeriodicity() {
        String periodicity = getRecurringSubscriptionUnit().charAt(0) + String.valueOf(getRecurringSubscriptionLength());
        if (periodicity.equals("m12")) {
            periodicity = "y1";
        }
        return periodicity;
    }

    private /*String*/ void getXml() {
/*        // Create an empty order
        LPOrderPart order = LPOrderFactory.createOrderPart("order");

        // Create a part we will use to build the order
        LPOrderPart op = LPOrderFactory.createOrderPart();

        // Build 'orderoptions'
        *//**
         * ordertype: The type of transaction. The possible values are SALE, PREAUTH (for an Authorize Only transaction), POSTAUTH (for a Forced Ticket or Ticket Only transaction), VOID, CREDIT, CALCSHIPPING (for shipping charges calculations) and CALCTAX (for sales tax calculations). See the Transaction Types section for additional information. Required
         *//*
        op.put("ordertype", getTransactionType());
        // live transaction
        *//**
         * result: This field puts the account in live mode or test mode. Set to LIVE for live mode, GOOD for an approved response in test mode, DECLINE for a declined reponse in test mode, or DUPLICATE for a duplicate response in test mode. Not required. Defaults to LIVE.
         *//*
        op.put("result", "LIVE");
        // add 'orderoptions to order
        order.addPart("orderoptions", op);

        // Build 'merchantinfo'
        op.clear();
        *//**
         * configfile: This field should contain the merchant store name or number, which is generally a six- to ten- digit number assigned at merchant account setup.
         * If this is a test account, the store name may be a text string. Required for ALL transactions and function calls.
         *//*
        op.put("configfile", login());
        // add 'merchantinfo to order
        order.addPart("merchantinfo", op);

        // Build 'transactiondetails'
        op.clear();
        *//**
         * The source of the transaction. The possible values are ECI (if the order was received via e-mail or Internet), MAIL (mail order), MOTO (mail order / telephone order),  TELEPHONE (telephone order) and RETAIL (face to face). For credit card transactions, mail order and telephone order transactions are treated the same. However, for VirtualCheck transactions, mail order and telephone order transactions are treated differently and the merchant should take care to identify them separately. Required for retail, mail, and telephone orders. Defaults to ECI.
         *//*
        op.put("transactionorigin", "ECI");
        // If there is no PO Number for this order,
        // pass a department code or other value, but make sure the value you pass is supplied by the customer
        op.put("ponumber", poNumber());
        // If the purchase is tax exempt,
        // pass a value of Y for taxexempt
        op.put("taxexempt", "N");
        op.put("terminaltype", "UNSPECIFIED");
        *//**
         * ip: The IP address of the consumer Not required, but needed for fraud blocking by IP address.
         *//*
        //op.put("ip", "123.123.123.123");
        *//**
         * oid: The Order ID to be assigned to this transaction. For SALE and PREAUTH, this field must be  unique. For VOID, CREDIT, and POSTAUTH, this field must be a valid Order ID from a prior SALE or PREAUTH transaction. Required for credit card VOID, CREDIT, and POSTAUTH transactions.
         *//*
        op.put("oid", getPreviousTransactionId());
        op.put("invoice_number", invoiceNumber());
        // add 'transactiondetails' to order
        order.addPart("transactiondetails", op);

        // Build 'billing'
        // Required for AVS. If not provided,
        // transactions will downgrade.
        op.clear();
        op.put("zip", zip());
        *//**
         * addrnum: The numeric portion of the street address Required for AVS
         *//*
        op.put("addrnum", removeNonIntegerChars(streetAddress()));

        op.put("name", cardholderName());
        //op.put("company", companyName()); // FIX!
        op.put("address1", address1());
//		op.put("address1", streetAddress());
        op.put("address2", address2());
        op.put("city", city());
        op.put("state", state());
        op.put("country", country());
        op.put("phone", phone());
//        op.put("fax", fax());
        op.put("email", customerEmail());
        op.put("userid", customerID());
        // add 'billing to order
        order.addPart("billing", op);

        // Build 'com.prosc.shipping'
        op.clear();
        op.put("name", shipFullName());
        op.put("address1", shipAddress());
        //op.put("address2", "Suite 23");
        op.put("city", shipCity());
        op.put("state", shipState());
        op.put("country", shipCountry());
        op.put("zip", shipZip());
        order.addPart("com.prosc.shipping", op);

        // Build 'creditcard'
        op.clear();
        op.put("cardnumber", cardNumber());
        op.put("cardexpmonth", expirationMonth());
        op.put("cardexpyear", expirationYear());
        op.put("cvmvalue", cardVerificationValue());
        op.put("cvmindicator", cardVerificationValue() == null ? "not_provided" : "provided");
        // optional track data for card-present transactions
        *//**
         * track: Contains the data swiped from the credit card track 1 or track 2. Required for credit card transactions if cardnumber, cardexpmonth and cardexpyear are not provided.
         *//*
        op.put("track", getTrack1() == null ? getTrack2() : getTrack1());
        // add 'creditcard to order
        order.addPart("creditcard", op);

        // Build 'payment'
        op.clear();
        //op.put("subtotal", subTotal()); // FIX!
        //op.put("tax", tax()); // FIX!
        //op.put("com.prosc.shipping", com.prosc.shipping()); FIX!
        //op.put("vattax", vatTax()); FIX!
        if (amount() != null) {
            op.put("chargetotal", amount().toString());
        }
        // add 'payment to order
        order.addPart("payment", op);

        // add notes to order
        op.clear();
//		op.put("referred", "Saw ad on Web site.");
//		op.put("comments", "Repeat customer. Ship immediately.");
        op.put("comments", chargeDescription());
        order.addPart("notes", op);

        // Create some parts we use to build order itmes
        LPOrderPart items = LPOrderFactory.createOrderPart();
        LPOrderPart item = LPOrderFactory.createOrderPart();
        LPOrderPart options = LPOrderFactory.createOrderPart();

//		// build and build 'item1'
//		item.put("id", "123456-A98765");
//		item.put("description", "Logo T-Shirt");
//		item.put("quantity", "1");
//		item.put("price", "12.99");
//		item.put("serial", "0987654321");
//
//		// build item's options
//		op.clear();
//		op.put("name", "Color");
//		op.put("value", "Red");
//		options.addPart("option", op, 1);
//
//		op.clear();
//		op.put("name", "Size");
//		op.put("value", "XL");
//		options.addPart("option", op, 2);
//
//		// add 'options' to item
//		item.addPart("options", options);
//
//		// add 'item' to 'items' collection
//		items.addPart("item", item, 1);
//		// add 'items' to order
//		order.addPart("items", items);

        return order.toXML();*/
    }

    protected String debugModeResponse() {
        return "<r_csp>CSI</r_csp>" +
                "<r_time>Thu Nov 6 04:14:15 2008</r_time>" +
                "<r_ref>0005441765</r_ref>" +
                "<r_error></r_error>" +
                "<r_ordernum>630E1B05-4912D186-584-1A42F2</r_ordernum>" +
                "<r_message>APPROVED</r_message>" +
                "<r_code>1234560005441765:NNN :100020617984:</r_code>" +
                "<r_tdate>1225970054</r_tdate>" +
                "<r_score></r_score>" +
                "<r_authresponse></r_authresponse>" +
                "<r_approved>APPROVED</r_approved>" +
                "<r_avs>NNN</r_avs>";
    }

    public void processResponse(String response) throws PaymentAuthorizationException {
        parseResponse(response);

    }

    /*
    /////////////////////////////////////////////////////////////////////////////
    //START OF NEW GETTERS AND SETTERS --svr
    /////////////////////////////////////////////////////////////////////////////
     */

    public BigDecimal getSubtotal() {
        return subtotal;
    }

    public void setSubtotal(BigDecimal subtotal) {
        if (subtotal != null) {
            this.subtotal = subtotal.setScale(2, BigDecimal.ROUND_HALF_UP);
        }
    }

    public BigDecimal getTax() {
        return tax;
    }

    public void setTax(BigDecimal tax) {
        if (tax != null) {
            this.tax = tax.setScale(2, BigDecimal.ROUND_HALF_UP);
        }
    }

    public BigDecimal getShipping() {
        return shipping;
    }

    public void setShipping(BigDecimal shipping) {
        if (shipping != null) {
            this.shipping = shipping.setScale(2, BigDecimal.ROUND_HALF_UP);
        }
    }

    public String getCompanyName() {
        return companyName;
    }

    public void setCompanyName(String companyName) {
        this.companyName = companyName;
    }

    public String getFax() {
        return fax;
    }

    public void setFax(String fax) {
        this.fax = fax;
    }

    public BigDecimal getWeight() {
        return weight;
    }

    public void setWeight(BigDecimal weight) {
        if (weight != null) {
            this.weight = weight.setScale(2, BigDecimal.ROUND_HALF_UP);
        }
    }

    public int getItems() {
        return items;
    }

    public void setItems(int items) {
        this.items = items;
    }

    public int getCarrier() {
        return carrier;
    }

    public void setCarrier(int carrier) {
        this.carrier = carrier;
    }

    public BigDecimal getTotal() {
        return total;
    }

    public void setTotal(BigDecimal total) {
        if (total != null) {
            this.total = total.setScale(2, BigDecimal.ROUND_HALF_UP);
        }
    }

    public int getTdate() {
        return tdate;
    }

    public void setTdate(int tdate) {
        this.tdate = tdate;
    }

    public String getReferred() {
        return referred;
    }

    public void setReferred(String referred) {
        this.referred = referred;
    }

    /*
   /////////////////////////////////////////////////////////////////////////////
   //END OF NEW GETTERS AND SETTERS --svr
   /////////////////////////////////////////////////////////////////////////////
    */

    public String tDate() {
        return R_TDate;
    }

    public String approvalCode() {
        return R_Code;
    }

    public String avsResponse() {
        return R_AVS;
    }

    public String resultCode() {
        return R_Approved;
//        return chargeResult;
    }

    public String transactionId() {
        return R_OrderNum;
    }

    public String getPreviousTransactionId() {
        return previousTransactionId;
    }

    public void setPreviousTransactionId(String previousTransactionId) {
        this.previousTransactionId = previousTransactionId;
    }

    public String getTransactionType() {
        return transactionType;
    }

    public void setTransactionType(String transactionType) {
        this.transactionType = transactionType;
    }

    public String getRecurringTransactionType() {
        return recurringTransactionType;
    }

    public void setRecurringTransactionType(String recurringTransactionType) {
        this.recurringTransactionType = recurringTransactionType;
    }

    public void setRecurringSubscriptionStartDate(Date recurringSubscriptionStartDate) {
        this.recurringSubscriptionStartDate = recurringStartDateFormat.format(recurringSubscriptionStartDate);
    }

    public Boolean getRecurring() {
        return isRecurring;
    }

    public void setRecurring(Boolean recurring) {
        isRecurring = recurring;
    }

    private String removeNonIntegerChars(String text) {
        if (text == null) return "";
        char[] chars = text.toCharArray();
        StringBuffer extracted = new StringBuffer(chars.length);
        for (int i = 0; i < chars.length; i++) {
            if (chars[i] >= '0' && chars[i] <= '9') {
                extracted.append(chars[i]);
            }
        }
        return (extracted.toString());
    }

    private KeyStore storeHostCertificate(String path, String alias) throws KeyStoreException, NoSuchAlgorithmException, IOException, CertificateException, KeyManagementException {

        InputStream trustedCertificateStream = getClass().getResourceAsStream(path);
        File tempKeyStoreFile = File.createTempFile("tmp", ".pem");
        FileOutputStream tempKeyStoreOutputStream = new FileOutputStream(tempKeyStoreFile);

        //Get a KeyStore of the default type (JKS)
        KeyStore tempKeyStore = KeyStore.getInstance(KeyStore.getDefaultType());

        try {

            //Get a CertificateFactory capable of generating X509 certificates
            CertificateFactory trustedCertificateFactory = CertificateFactory.getInstance("X509");

            //Generate a X509Certificate from the resource in the classpath
            X509Certificate trustedCertificate = (X509Certificate) trustedCertificateFactory.generateCertificate(trustedCertificateStream);

            //Initialize keystore
            tempKeyStore.load(null, "changeit".toCharArray()); //second parameter is a password for accessing the keystore

            //Add the X509Certificate to the KeyStore
            tempKeyStore.setCertificateEntry(alias, trustedCertificate);
//            tempKeyStore.setCertificateEntry("linkpoint", trustedCertificate);

            //Store the KeyStore as a File
            tempKeyStore.store(tempKeyStoreOutputStream, "changeit".toCharArray());

            //Set the TrustStore to the temporary KeyStore
            System.setProperty("javax.net.ssl.trustStore", tempKeyStoreFile.getAbsolutePath());
            System.setProperty("javax.net.ssl.trustStorePassword", "changeit");
        } finally {
            tempKeyStoreOutputStream.close();
            trustedCertificateStream.close();
        }

        return tempKeyStore;
    }

    public void setTestMode(boolean testMode) {
        this.testMode = testMode;
    }

    public boolean isTestMode() {
        return testMode;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public File getCertPath() {
        return certPath;
    }

    public void setCertPath(File certPath) {
        this.certPath = certPath;
    }

    public String getBankState() {
        return bankState;
    }

    public void setBankState(String bankState) {
        this.bankState = bankState;
    }

    public String getDriversLicenseNumber() {
        return driversLicenseNumber;
    }

    public void setDriversLicenseNumber(String driversLicenseNumber) {
        this.driversLicenseNumber = driversLicenseNumber;
    }

    public String getDriversLicenseState() {
        return driversLicenseState;
    }

    public void setDriversLicenseState(String driversLicenseState) {
        this.driversLicenseState = driversLicenseState;
    }

    public String getTender() {
        return tender;
    }

    public void setTender(String tender) {
        this.tender = tender;
    }

    public void clear() {
        if (certPath != null /*&& !certPath.delete()*/) {
//            log.warning("Unable to delete converted certificate at " + certPath.getAbsolutePath());
            certPath.deleteOnExit();
        }
        System.clearProperty("javax.net.ssl.trustStore");
        R_TDate = "";
        R_AVS = "";
        R_Code = "";
        R_OrderNum = "";
        chargeResult = 0;
    }
}



